unit vga;
{
 False VGA support for Delphi/Windows (c)2001 by Paul TOTH <tothpaul@free.Fr>
 http://tothpaul.free.Fr
}
Interface

uses
 windows,messages;

procedure GraphMode;
procedure TextMode;

procedure ClearScreen;
procedure SetColor(i,r,g,b:integer);
procedure SetPalette(var colors);
procedure PutPixel(x,y,c:integer);
procedure Retrace;

Function GetScreen:pointer;
Procedure SetScreen(p:pointer);

implementation

var
 pixels:packed array[0..199,0..319] of byte;

function GetScreen:pointer;
 begin
  GetMem(result,320*200);
  Move(pixels,Result^,320*200);
 end;

procedure SetScreen(p:pointer);
 begin
  Move(p^,pixels,320*200);
  FreeMem(p);
 end;

const
 ClassName='VGA256';

Type
 TPalette256=packed record
  Version:word;
  Entries:word;
  Colors:array[0..255] of integer;
 end;

 TBitmapInfo=packed record
  Header:TBitmapInfoHeader;
  Colors:Array[0..255] of Word;
 end;

var
 win:THandle;
 hdc:THandle;
 pal:THandle;
 Border,Caption:integer;
 Width,Height:integer;
 BitmapInfo :TBitmapInfo;

procedure Paint;
 var
  old:THandle;
 begin
  Old:=SelectPalette(hdc,pal,False);
  RealizePalette(hdc);
  StretchDiBits(hdc, 0,0,Width,Height, 0,0,320,200, @Pixels,PBitmapInfo(@BitmapInfo)^, DIB_PAL_COLORS, SRCCOPY);
  SelectPalette(hdc,Old,False);
 end;

Function WinProc(hWnd: THandle; Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
 begin
  Result:=DefWindowProc(hWnd, msg, wParam, lParam);
  Case Msg of
   WM_SIZE    : begin
                 Width :=LOWORD(lParam);
                 Height:=HIWORD(lParam);
                end;

   WM_PAINT   : Paint;

   WM_DESTROY : PostQuitMessage(0);
  end;
 end;

procedure GraphMode;
 var
  wc :TWndClass;
  i  :integer;
  palette:TPalette256;  // Color palette
 begin
 { Register the window class for my window. }
  wc.style :=  0; // CS_SAVEBITS;            { Class style. }
  wc.lpfnWndProc := @WinProc;          { Window procedure for this class. }
  wc.cbClsExtra := 0;                  { No per-class extra data. }
  wc.cbWndExtra := 0;                  { No per-window extra data. }
  wc.hInstance := hInstance;           { Application that owns the class. }
  wc.hIcon := LoadIcon(hInstance,'MAINICON');  { Delphi icon }
  wc.hCursor := LoadCursor(0, IDC_ARROW); { defaut cursor }
  wc.hbrBackground := 0; // GetStockObject(BLACK_BRUSH); { black screen }
  wc.lpszMenuName := nil; // 'MainMenu';       { Name of menu resource in .RC file. }
  wc.lpszClassName := ClassName;      { Name used in call to CreateWindow. }
  if RegisterClass(wc)=0 then halt;

  With BitmapInfo do begin
   with header do begin
    biSize         :=SizeOf(TBitmapInfoHeader);
    biWidth        :=320;
    biHeight       :=-200;
    biplanes       :=1;
    bibitcount     :=8; { for True-Color }
    bicompression  :=BI_RGB;
    biSizeImage    :=320*200;
    bixpelspermeter:=0;
    biypelspermeter:=0;
    biclrused      :=256;
    biclrimportant :=0;
   end;
   For i:=0 to 255 do Colors[i]:=i; { cause we're using a dib_pal_colors }
  end;

  Border :=2*(GetSystemMetrics(SM_CXDLGFRAME)+GetSystemMetrics(SM_CXBORDER));
  Caption:=GetSystemMetrics(SM_CYCAPTION);
  win:= CreateWindow(
   ClassName,  { registered class name }
   'VGA for Windows',
   WS_OVERLAPPEDWINDOW or ws_visible,   { Window style. }
   (GetSystemMetrics(SM_CXFULLSCREEN)-320) div 2, // cw_usedefault,         { Default horizontal position. }
   (GetSystemMetrics(SM_CYFULLSCREEN)-200) div 2, // cw_usedefault,         { Default vertical position. }
   320+Border,
   200+Border+Caption,
   0,                     { Overlapped windows have no parent. }
   0,                     { Use the window class menu. }
   hInstance,             { This instance owns this window. }
   nil                    { Pointer not needed. }
  );
  if win=0 then halt;
  hdc:=GetDC(win);

  Palette.Version:=$300;
  Palette.Entries:=256;
  GetSystemPaletteEntries(GetDC(0),0,256,Palette.Colors);
  pal:=CreatePalette(PLogPalette(@Palette)^);
 end;

procedure TextMode;
 begin
  DeleteObject(pal);
  ReleaseDC(win,hdc);
  DestroyWindow(win);
 end;

procedure ClearScreen;
 begin
  fillchar(pixels,320*200,0);
 end;
procedure SetColor(i,r,g,b:integer);
 var
  color:integer;
 begin
  color:=(((((PC_NOCOLLAPSE shl 8) + b) shl 8) + g) shl 8) + r;
  SetPaletteEntries(pal,i,1,Color);
 end;

function BGR2RGB(i:integer):integer;
//asm bswap eax; shl eax,8 end;
 begin
  result:=(i and $FF) shl 16+(i and $FF00FF00)+(i shr 16) and $FF;
 end;

procedure SetPalette(var colors);
 var
  i:integer;
  c:^integer;
 begin
  c:=@colors;
  for i:=0 to 255 do begin
   c^:=bgr2rgb(c^);
   inc(c);
  end;
  SetPaletteEntries(pal,0,256,Colors);
 end;

procedure PutPixel(x,y,c:integer);
 begin
  Pixels[y,x]:=c;
 end;

procedure DoEvents;
 var
  msg:TMsg;
 begin
  while PeekMessage(Msg,0,0,0,pm_remove) do begin
   TranslateMessage(msg);    { Translates virtual key codes. }
   DispatchMessage(msg);     { Dispatches message to window. }
   if msg.Message=WM_QUIT then begin
    TextMode;
    halt;
   end;
  end;
 end;
 
Procedure Retrace;
 begin
  Paint;
  DoEvents;
 end;

end.
